/* mms_pdu.h - definitions for unpacked WTP protocol data units
 *
 */

#ifndef MMS_PDU_H
#define MMS_PDU_H

#include "gwlib/gwlib.h"

#define MMS_TRUE_TOKEN 128
#define MMS_FALSE_TOKEN 129

/* Enumerate the symbolic names of the PDUs */
enum mms_pdu_types {

#define PDU(name, pdutype, docstring, fields, is_valid) name = pdutype,
#include "mms_pdu.def"
#undef PDU
};

struct mms_pdu {
	int type;

	union {
/* For each PDU, declare a structure with its fields, named after the PDU */

#define PDU(name, pdutype, docstring, fields, is_valid) struct name { fields } name;
#define UINT(field, fieldnr, docstring) unsigned int field;
#define MMSVERSION(field, fieldnr, docstring) Octstr *field;
#define OCTSTR(field, fieldnr, docstring) Octstr *field;
#define OCTSTRE(field, fieldnr, docstring) Octstr *field;
#define ULONG(field, fieldnr, docstring) long field;
#define ULONGABSREL(field, fieldnr, docstring) long field; \
                                               int field ## _absrelind;
#define ENCSTRVAL(field, fieldnr, docstring) Octstr *field; \
                                               unsigned long field ## _charset;
#define ENCSTRVALE(field, fieldnr, docstring) Octstr *field; \
                                               unsigned long field ## _charset;
#define REST(field, docstring) Octstr *field;
#include "mms_pdu.def"
#undef REST
#undef ENCSTRVAL
#undef ENCSTRVALE
#undef ULONG
#undef ULONGABSREL
#undef OCTSTR
#undef OCTSTRE
#undef MMSVERSION
#undef UINT
#undef PDU
	} u;
};
typedef struct mms_pdu MMS_PDU;

MMS_PDU *mms_pdu_create(int type);
MMS_PDU *mms_pdu_unpack(Octstr *data);
Octstr *mms_pdu_pack(MMS_PDU *pdu);
Octstr *mms_pdu_to_xml(MMS_PDU *pdu);
void mms_pdu_destroy(MMS_PDU *pdu);
void mms_pdu_destroy_item(void *pdu);


#endif
